 
using System.IO;
using System;
using System.Collections;

 
namespace Libregresion{

class MonuConsole{
	
	static double[] xarr; // x data 
	static double[] yarr;
	static bool saved = false; 
 
	// Main method for Console versions of the program.
#if (CONSOLE)
	public static void Main ()
	{
		Console.WriteLine ("\n\n\t\t\tReAgresion 1.0\n\n");
		Console.WriteLine ("\tThe definitive calculator for physicists and\n" +
				   "\tthe people that need to do lots of regressions,\n" +
				   "\tinterpolations, solving systems...\n");
		Console.WriteLine ("\tDistributed under the GNU General Public License v 2.0\n");
		Console.WriteLine ("\t(c) 2003, Jaime Anguiano Olarra <jaime@gnome.org>\n\n");

		bool yep = true;
		
		while (yep) {
			PrintMenu ();
			switch (Console.ReadLine()) {
			case "1":
				DoLibregresion ();
				break;
			case "2":
				DoInterpolation ();
				break;
			case "3":
				DoGaussSeidel ();
				break;
			case "4":
				Save ();
				break;
			case "5":
				Modify ();
				break;
			case "6":
				Show ();
				break;
			case "7":
				if (!saved) {
					Console.WriteLine ("Your data hasn't been saved!\n" +
							   "Do you want to save it now? (y/n)");
					if (String.Compare ((Console.ReadLine()).ToUpper(),"Y") == 0) {
						Save ();
						saved = true;
					}
				}
					
				yep = false;
				break;
			}
		}
	}
#endif 
	
	public static void PrintMenu ()
	{
		Console.WriteLine ("\n\nEnter your choice: ");
		Console.WriteLine ("\t1. Libregresion");
		Console.WriteLine ("\t2. Interpolation");
		Console.WriteLine ("\t3. Gauss-Seidel");
		Console.WriteLine ("\t4. Save data");
		Console.WriteLine ("\t5. Modify data");
		Console.WriteLine ("\t6. Show data");
		Console.WriteLine ("\t7. Quit\n\n");
	}
	
	public static void DoLibregresion ()
	{
		int ord;
		long itr;
		double[] sols;
		
		AskForInput ();
		Console.Write ("Enter the order of the polinomy to use: ");
		ord = Convert.ToInt32 (Console.ReadLine ()) + 1;
		Console.WriteLine ();
		Console.WriteLine ("Note: This program uses the Gauss-Seidel method by default to calculate\n" +
				   "the polinomy coefficients. Normally you should get a nice convergency and\n" +
				   "you shouldn't enter numbers above 15000.\n\n");
		Console.Write ("Enter the number of iterations: ");
		itr = (long) Convert.ToDouble (Console.ReadLine ());

		sols = Regresion.Regres (xarr, yarr, ord, itr);
		
		ShowSols (sols);
		Console.Write ("Save results? (y/n): ");
		if (String.Compare ((Console.ReadLine ()).ToUpper(), "Y") == 0) {
			Console.Write ("Enter the file to use: ");
			StreamWriter sw = new StreamWriter (Console.ReadLine());
			sw.WriteLine ("Results obtained by regresion:\n\nThe vectors entered:\n(x data)");
			WriteVector (sw, xarr);
			sw.WriteLine ("(y data):");
			WriteVector (sw, yarr);
			sw.WriteLine ("The Gauss-Seidel solutions for them after " + itr + " iterations:");
			WriteVector (sw, sols);
			sw.Flush ();
			sw.Close ();
			sw = null;
			saved = true;
		}
	}

	public static void DoInterpolation ()
	{
		double xval;
		double[] results;
		AskForInput ();
		Console.Write ("Enter the value for the one you want to interpolate (x): ");
		xval = Convert.ToDouble (Console.ReadLine());
		results = Regresion.Interpolation (xarr, yarr, xval);
		Show (results);
	}

	public static void DoGaussSeidel ()
	{
		string s = null;
		int order;
		double[,] matrix;
		double[] sols;
		double[] gs_sols;
		int iter;
		
		Console.Write ("Enter the number of rows: ");
		order = Convert.ToInt32 (Console.ReadLine ());
		matrix = new double[order, order];
		Console.WriteLine ("\nEnter the matrix elements:\n");
		for (int i=0; i < order; i++) {
			for (int j=0; j < order; j++) {
				Console.Write ("a" + i + j + ": ");
				matrix[i,j] = Convert.ToDouble (Console.ReadLine());
			}
		}

		Console.WriteLine ("\nEnter the solution vector:");
		sols = new double[order];
		for (int k=0; k < order; k++) {
			Console.Write ("s" + k + ": ");
			sols[k] = Convert.ToDouble (Console.ReadLine ());
		}

		Console.Write ("Enter the number of iterations to use: ");
		iter = Convert.ToInt32 (Console.ReadLine ());
		gs_sols = Regresion.GaussSeidel (matrix, sols, iter);
		ShowSols (gs_sols);
		saved = false;
		Console.Write ("Save results? (y/n): ");
		if (String.Compare ((Console.ReadLine ()).ToUpper(), "Y") == 0) {
			Console.Write ("Enter the file to use: ");
			StreamWriter sw = new StreamWriter (Console.ReadLine());
			sw.WriteLine ("Results obtained by Gauss-Seidel:\n\nThe matrix entered:");
			WriteMatrix (sw, matrix, order);
			sw.WriteLine ("The solution vector entered:");
			WriteVector (sw, sols);
			sw.WriteLine ("The Gauss-Seidel solutions for them after " + iter + " iterations:");
			WriteVector (sw, gs_sols);
			sw.Flush ();
			sw.Close ();
			sw = null;
			saved = true;
		}
	}

	public static void Save ()
	{
		StreamWriter sw = new StreamWriter ("data.txt");
		sw.WriteLine ("X Data:\n");
		for (int i=0; i < xarr.Length; i++)
			sw.WriteLine ("x" + i + ": " + xarr[i]);
		sw.WriteLine ("\nY Data:\n");
		for (int j=0; j < yarr.Length; j++)
			sw.WriteLine ("y" + j + ": " + yarr[j]);
		sw.Flush ();
		sw.Close ();
		sw = null;
		saved = true;
	}

	public static void WriteVector (StreamWriter sw, double[] vctr)
	{
		for (int i=0; i < vctr.Length; i++)
			sw.WriteLine ("s" + i + ": " + vctr[i]);
		sw.WriteLine ();
	}

	public static void WriteMatrix (StreamWriter sw, double[,] mtrx, int order)
	{
		for (int i=0; i < order; i++) {
			for (int j=0; j < order; j++) {
				sw.Write (mtrx[i,j] + " ");
				if (j == order - 1)
					sw.WriteLine ();
			}
		}
		sw.WriteLine ();
	}
	
	public static void Modify ()
	{
		Console.Write ("\n\nDo you want to modify an 'X' or an 'Y' (x/y)?: ");
		if (String.Compare ((Console.ReadLine ()).ToUpper(), "X") == 0) {
			modifyx:
			Console.Write ("\nEnter the index (starting at 0): ");
			int i = Convert.ToInt32 (Console.ReadLine());
			Console.WriteLine ("The current value of x" + i + " is: " + xarr[i]);
			Console.Write ("\nEnter the new value for x" + i + ": ");
			xarr[i] = Convert.ToDouble (Console.ReadLine());
			Console.WriteLine ("Modify another value?: (y/n)");
			if (String.Compare ((Console.ReadLine ()).ToUpper(), "Y") == 0)
				goto modifyx;
		} else {
			modifyy:
			Console.Write ("\nEnter the index (starting at 0): ");
			int j = Convert.ToInt32 (Console.ReadLine());
			Console.WriteLine ("The current value of y" + j + " is: " + yarr[j]);
			Console.Write ("\nEnter the new value for y" + j + ": ");
			yarr[j] = Convert.ToDouble (Console.ReadLine());
			Console.WriteLine ("Modify another value?: (y/n)");
			if (String.Compare ((Console.ReadLine ()).ToUpper(), "Y") == 0)
				goto modifyy;
		}

	}

	public static void Show ()
	{
		Console.WriteLine ("\nThe current data values are:");
		Console.WriteLine ("\nX\t\tY\n");
		for (int i=0; i < xarr.Length; i++) {
			Console.WriteLine (xarr[i] + "\t\t" + yarr[i]);
		}
	}
	
	// Asks for X/Y style input, that is, it allows you to retrieve the x_data
	// followed by the y_data
	public static void AskForInput ()
	{
		ArrayList xa = new ArrayList();
		saved = false;
		string s = null;
		bool yep = true;
		int i = 0;
		Console.WriteLine ("Enter the independent variable (X) points: ");
		Console.WriteLine ("(enter 'q' to stop)");
		while (yep) {
			Console.Write ("x" + i + ": ");
			s = Console.ReadLine ();
			if (String.Compare (s, "q") == 0) {
				yep = false;
				continue;
			}
			xa.Add (Convert.ToDouble(s));
			i++;
		}

		xarr = new double[xa.Count];
		yarr = new double[xa.Count];
		
		Console.WriteLine ("Enter the dependent variable (Y) points: ");
		for (int j=0; j < xa.Count; j++) {
			Console.Write ("y" + j + ": ");
			s = Console.ReadLine ();
			yarr[j] = Convert.ToDouble(s);
		}

		for (int j=0; j < xa.Count; j++) {
			xarr[j] = (double) xa[j];
		}
	}
	
	// Prints the content of a matrix of n rows, m columns to the
	// console.
	public static void Show (double[,] matrix, int n, int m)
	{
		for (int i=0; i < n; i++) {
			for (int j=0; j < m; j++) {
				Console.Write (matrix[i,j] + " ");
			}
			Console.WriteLine ();
		}
		Console.WriteLine ();
	}

	// Prints the content of a solution array with an improved
	// look.
	public static void ShowSols (double[] arr)
	{
		for (int i=0; i < arr.Length; i++)
			Console.WriteLine ("x" + i + ": {0}", arr[i]);
	}
	
	// Prints the content of an array to the console.
	public static void Show (double[] arr)
	{
		for (int i=0; i < arr.Length; i++) 
			Console.Write (arr[i] + " ");
		Console.WriteLine ();
	}
}

}