using System;

namespace Libregresion{

	public struct Complex{
		public double[] val;
		public Complex( double re, double im)
		{
			val = new double[2];
			this.val[0]= re;
			this.val[1] = im;
		}

		public double Re{
			get{ return this.val[0]; }
			set{ this.val[0] = value;}
		}
		public double Im{
			get{ return this.val[1]; }
			set{ this.val[1] = value;}
		}
		
		public double Norm{
			get{ 
				return Math.Sqrt((this.val[0]*this.val[0])
						+ (this.val[1]*this.val[1]) );
			}
		}

		public static bool operator ==( Complex z, Complex w)
		{
			return ((z.val[0] == w.val[0]) && (z.val[1] == w.val[1]));
		}

		public static bool operator != ( Complex z, Complex w)
		{
			return ((z.val[0] != w.val[0]) || (z.val[1] != w.val[1]));
		}

		public override bool Equals( object o )
		{
			if( !( o is Complex))
				return false;
			return this == ((Complex) o);
		}

		public static Complex operator *(Complex z, Complex w)
		{
			return  new Complex(z.val[0]*w.val[0] - z.val[1]*w.val[1],
					z.val[0]*w.val[1] + z.val[1]*w.val[0]);
		}

		public static Complex operator *(int n, Complex z)
		{
			return new Complex(n*z.val[0], n*z.val[1]);
		}

		public static Complex operator *(Complex z, int n)
		{	
			return new Complex(n*z.val[0], n*z.val[1]);
		}


		public static Complex operator /(Complex z, Complex w)
		{
			double a = w.val[0]*w.val[0] + w.val[1]*w.val[1];
			return  new Complex((z.val[0]*w.val[0] + z.val[1]*w.val[1])/a ,
					(-z.val[0]*w.val[1] + z.val[1]*w.val[0])/a);
		}

		public static Complex operator +( Complex z, Complex w)
		{
			return new Complex(z.val[0] + w.val[0] , z.val[1] + w.val[1]);
		}

		public static Complex operator -( Complex z, Complex w)
		{
			return new Complex(z.val[0] - w.val[0] , z.val[1] - w.val[1]);
		}
		
		public static Complex operator -(Complex z)
		{
			z.val[0] = -z.val[0];
			z.val[1] = -z.val[1];
			return z;
		}

		public override String ToString()
		{
			return String.Format( "{0} + {1}i", this.val[0], this.val[1] );
		}

		public override int GetHashCode()
		{       
			return base.GetHashCode ();
		}
	}
}


