

namespace Libregresion{

	public class MatrixAlgebra{
		
	
	public static double Trace( double[,] A )
	{
		int n = A.GetLength(0);
		if( n!= A.GetLength(1) )
			throw new LibregresionException(" Not Square Matrix");
		double tr = 0;
		for(int i = 0; i< n; i++)
			tr += A[i,i];
		return tr;
	}

	public static Complex Trace( Complex[,] A)
	{
		int n = A.GetLength(0);
		if( n!= A.GetLength(1) )
			throw new LibregresionException(" Not Square Matrix");
		Complex tr = new Complex(0,0);
		for(int i = 0; i< n; i++)
			tr += A[i,i];
		return tr;
	}

	

	
	public static double[,] Transpose( double[,] M, int n )
	{
		if( n != M.GetLength(1) )
			throw new LibregresionException("Not square matrix");
		double[,] trans = new double[n,n];
		for( int i = 0; i<n; i++){
			for( int j = 0; j<n; j++){
				trans[i,j] = M[j,i];
			}
		}
		return trans;
	}

	
	public static Complex[,] Transpose( Complex[,] M , int n)
	{
		if( n != M.GetLength(1) )
			throw new LibregresionException("Not square matrix");
		Complex[,] trans = new Complex[n,n];
		for( int i = 0; i<n; i++){
			for( int j = 0; j<n; j++){
				trans[i,j] = M[j,i];
			}
		}
		return trans;
	}

	
	// Returns an array which each element is calculated by
	// multiplying each the corresponding elements from the
	// two arrays passed to it.
	public static double[] Multiply (double[] xarr, double[] yarr)
	{
		double[] zarr = new double[xarr.Length];
		
		for (int i=0; i < xarr.Length; i++) {
			zarr[i] = xarr[i] * yarr[i];
		}

		return zarr;
	}


	// Returns the matrix generated by multiplying a * b
	public static double[,] Multiply (double[,] a, double[,] b, uint n)
	{
		double[,] c;
		double y;

		c = new double[n,n];
			
		for (uint i=0; i < n; ++i) {
			for (uint j=0; j < n; ++j) {
				y = 0;
				for (uint k=0; k < n; ++k)
					y += a[i,k]*b[k,j];
				c[i,j] = y;
			}
		}

		return c;
	}


	// Returns the vector obtained by multiplying m (matrix) for v (vector)
	public static double[] Multiply (double[,] m, double[] v)
	{
		double y;
		double[] s;

		s = new double[v.Length];
		
		for (uint i=0; i < v.Length; ++i) {
			y = 0;
			for (uint j=0; j < v.Length; ++j)
				y += m[i,j]*v[j];
			s[i] = y;
		}

		return s;
	}





	
	// Returns the adjoint matrix of a complete matrix
	public static double[,] Adjoint (double[,] x, int order)
	{
		double[,] matrix;

		if( order == 1 ){
			return x;
		} else if( order == 2 ){
			matrix = new double[2,2];
			matrix[0,0] = x[1,1];
			matrix[0,1] = -x[0,1];
			matrix[1,0] = -x[1,0];
			matrix[1,1] = x[0,0];
			return matrix;
		}	// solucionado problema: no funcionaba para matrices de orden 2 ( devolvía matriz de ceros )
			// <- pero modificas la matrz que pasas ( los arrays se toman por referencia )

		matrix = new double[order, order];

		for (int i=0; i < order; ++i)
			for (int j=0; j < order; ++j) 
				matrix[i,j] = Adjoint(x, i, j, order);
		return matrix;
	}

public static Complex[,] Adjoint (Complex[,] x, int order)
	{
		Complex[,] matrix;

		if( order == 1 ){
			return x;
		} else if( order == 2 ){
			matrix = new Complex[2,2];
			matrix[0,0] = x[1,1];
			matrix[0,1] = -x[0,1];
			matrix[1,0] = -x[1,0];
			matrix[1,1] = x[0,0];
			return matrix;
		}	// solucionado problema: no funcionaba para matrices de orden 2 ( devolvía matriz de ceros )
			// <- pero modificas la matrz que pasas ( los arrays se toman por referencia )

		matrix = new Complex[order, order];

		for (int i=0; i < order; ++i)
			for (int j=0; j < order; ++j) 
				matrix[i,j] = Adjoint(x, i, j, order);
		return matrix;
	}

	public static double Adjoint (double[,] x, int n, 
			int m, int order)
	{
		return (Regresion.Sign(n+m) *
			LinearAlgebra.Det(AdjointMatrix(x, n, m, order)));
	}
	
	public static Complex Adjoint (Complex[,] x, int n, 
			int m, int order)
	{
		return (Regresion.Sign(n+m) *
			LinearAlgebra.Det(AdjointMatrix(x, n, m, order)));
	}



	public static double[,] AdjointMatrix (double[,] x,
			int n, int m, int order)
	{
		double[,] adj;
		uint r = 0, c = 0;
			
		
		adj = new double[order - 1, order - 1];
		for (uint i=0; i < order; ++i) {
			if (i == n)
				continue;
			for (uint j=0; j < order; ++j) 
			{
				if (c == (order - 1)) { 
					++r;
					c = 0;
				}
				if (j == m) 
					continue;
				adj[r,c] = x[i,j];
				++c;
			}
		}
		return adj;
	}


	public static Complex[,] AdjointMatrix (Complex[,] x,
			int n, int m, int order)
	{
		Complex[,] adj;
		uint r = 0, c = 0;
			
		
		adj = new Complex[order - 1, order - 1];
		for (uint i=0; i < order; ++i) {
			if (i == n)
				continue;
			for (uint j=0; j < order; ++j) 
			{
				if (c == (order - 1)) { 
					++r;
					c = 0;
				}
				if (j == m) 
					continue;
				adj[r,c] = x[i,j];
				++c;
			}
		}
		return adj;
	}


	}
}
			

