//
// MoNumeric -- A simple Gtk# frontend for the libregresion
// library.
//
// (c) 2003, Jaime Anguiano Olarra <jaime@gnome.org>
//           Fabian Seoane <f.seoane@unix.net>

using System;
using System.IO;
using Libregresion;
using Gdk;
using Gtk;
using GtkSharp;
using Gnome;
using Glade;
using GladeSharp;
using System.Drawing;
using System.Diagnostics;
using System.Reflection;
using System.Threading;

public class MonuMeric {
	const string VERSION = "0.1";
	const string ALIAS = "The power of Gtk#";
	static Glade.XML ui;
	static Gtk.Window window1;
	static Gtk.Entry xentry;
	static Gtk.Entry yentry;
	static Gtk.Notebook notebook;
	static TextView textview;
	static TextBuffer terminal;
	static Button regresButton;
	static Button interpButton;
	static Gtk.Entry orderEntry;
	static Gtk.Entry iterEntry;
	static Gtk.Entry interp_xentry;
	static Gtk.Entry interp_yentry;
	static Gtk.Entry interp_val_entry;
	static Gtk.Entry interp_results;
	static Gtk.Entry aprox_entry;
	static Gtk.Entry poly_sol;
	static TextView systemsInput;
	static TextView systemsOutput;
	static Gtk.Entry systemsIters;
	static Gnome.Canvas canvas;
	static double[,] matrix;
	static Complex[,] cmatrix;
	static Complex[] csolVctr;
	static double[] xarr;
	static double[] yarr;
	static double[] poly;
	static double aprox;
	static int order;
	static int iters;
	static Process gcalc = null;
	static double[] regsols;
	static ToggleButton autosaveButton;
	static bool[] changes;
	static Gtk.Dialog about;
	static bool flag = true;
	
	
	public MonuMeric ()
	{
		ui = new Glade.XML (null, "ui.glade", "window1", null);
		ui.Autoconnect (this);

		LoadPointers();
	}

	private static void LoadPointers ()
	{
		window1 = (Gtk.Window) ui["window1"];
		window1.SetDefaultSize (500, 500);
		window1.ShowAll();
		notebook = (Gtk.Notebook) ui["notebook1"];
		xentry = (Gtk.Entry) ui["entry1"];
		yentry = (Gtk.Entry) ui["entry2"];
		orderEntry = (Gtk.Entry) ui["order_entry"];
		iterEntry = (Gtk.Entry) ui["iter_entry"];
		textview = (Gtk.TextView) ui["textview1"];
		terminal = textview.Buffer;
		Gtk.Image image = (Gtk.Image) ui["image1"];
		image.Pixbuf = new Gdk.Pixbuf ("icon.png");
		Gtk.Label version = (Gtk.Label) ui["version_label"];
		version.Text = VERSION;
		Gtk.Label alias = (Gtk.Label) ui["alias_label"];
		alias.Text = ALIAS;
		interp_xentry = (Gtk.Entry) ui["interp_xentry"];
		interp_yentry = (Gtk.Entry) ui["interp_yentry"];
		interp_val_entry = (Gtk.Entry) ui["interp_val_entry"];
		interp_results = (Gtk.Entry) ui["interp_results"];
		systemsInput = (Gtk.TextView) ui["systems_input"];
		systemsOutput = (Gtk.TextView) ui["systems_output"];
		systemsIters = (Gtk.Entry) ui["systems_iters"];
		regresButton = (Gtk.Button) ui["regres_button"];
		interpButton = (Gtk.Button) ui["interp_button"];
		canvas = (Gnome.Canvas) ui["plot_canvas"];
		autosaveButton = (Gtk.ToggleButton) ui["autosave_button"];
		changes = new bool[3];
	}

	public static void Main ()
	{
		Application.Init();
		MonuMeric app = new MonuMeric ();
		Application.Run();
	}

	private static void on_delete_event (object o, DeleteEventArgs args)
	{
		Application.Quit();
	}

	private static void on_regres_clicked (object o, EventArgs args)
	{
		int c = 0;

		Clear ();
		GetInput (xentry, yentry);
		regsols = Regresion.Regres (xarr, yarr, order, iters);
		while (c < regsols.Length) 
			terminal.Insert (terminal.EndIter, "x" + (c).ToString() 
					+ ": " + (regsols[c++]).ToString() + "\n");
		changes[0] = true;
		if (autosaveButton.Mode == true)
			save ();
	}

	private static void on_interp_clicked (object o, EventArgs args)
	{
		double[] sols;
		double ival;
		
		GetInput (interp_xentry, interp_yentry);
		ival = Convert.ToDouble (interp_val_entry.Text);
		sols = Regresion.Interpolation (xarr, yarr, ival);
		interp_results.Text = (sols[0]).ToString();
		changes[1] = true;
		if (autosaveButton.Mode == true)
			save ();
	}

	private static void GetInput (Gtk.Entry xentry, Gtk.Entry yentry)
	{
		string s1, s2;
		string[] xdata, ydata;
		s1 = xentry.Text;
		s2 = yentry.Text;

		xdata = s1.Split(' ');
		ydata = s2.Split(' ');
		if (xdata.Length != ydata.Length)
			throw new Exception ();
		xarr = new double[xdata.Length];
		yarr = new double[ydata.Length];
		for (int i=0; i < xdata.Length; ++i) {
			xarr[i] = Convert.ToDouble(xdata[i]);
			yarr[i] = Convert.ToDouble(ydata[i]);
		}

		order = Convert.ToInt32 (orderEntry.Text);
		++order;
		iters = Convert.ToInt32 (iterEntry.Text);
	}

	private static void on_ryentry_changed (object o, EventArgs args)
	{
		string xd, yd;
		xd = xentry.Text;
		yd = yentry.Text;
		if (yentry.Text.Length > 2)
			((Gtk.Label) ui["ryel_label"]).Text = ((xd.Split(' ')).Length 
						      - (yd.Split(' ')).Length).ToString();
	}
	
	private static void Clear ()
	{
		terminal.Delete (terminal.StartIter, terminal.EndIter);
	}


/****************************************************
 * 		MATRICES
 ****************************************************/


	private static void on_matrix_clear( object o, EventArgs args)
	{
		TextBuffer obuffer = systemsOutput.Buffer;
		obuffer.Clear();
	}


	private static void on_systems_gs_clicked (object o, EventArgs args)
	{
		double[] solVctr;
		double[] sols;
		double iter;
		TextBuffer ibuffer = systemsInput.Buffer;
		TextBuffer obuffer = systemsOutput.Buffer;

		try {
			ProcessSystemInput (out matrix,out cmatrix, out flag, out solVctr, out csolVctr);

			iter = Convert.ToDouble (systemsIters.Text);
			sols = Regresion.GaussSeidel (matrix, solVctr, (long) iter);

			obuffer.Insert (obuffer.EndIter, "Solutions obtained (*)\n");
		
			for (int i=0; i < sols.Length; ++i)
				obuffer.Insert (obuffer.EndIter, (sols[i]).ToString() + "\n");
		
		} catch( Exception e ){
			Console.WriteLine( e.ToString() );
		}
		
	}

	private static void on_mvpm_clicked (object o, EventArgs args)
	{
	}

	

	private static void on_systems_gj_clicked (object o, EventArgs args)
	{
		Complex[,] cmatrix;
		double[] solVctr; 
		double[] sols;
		Complex[] csols;
		TextBuffer ibuffer = systemsInput.Buffer;
		TextBuffer obuffer = systemsOutput.Buffer;

		try {
			ProcessSystemInput (out matrix, out cmatrix, out flag, out solVctr, out csolVctr);
			
			obuffer.Insert (obuffer.EndIter, "Solutions obtained (*)\n");
	
			if( flag ){	
				sols = LinearAlgebra.Solve(matrix, solVctr);
				for (int i=0; i < sols.Length; ++i)
					obuffer.Insert (obuffer.EndIter, (sols[i]).ToString() + "\n");
			} else {
				csols = LinearAlgebra.Solve( cmatrix, csolVctr );
				for( int i = 0; i< csols.Length; ++i)
				obuffer.Insert( obuffer.EndIter, ( csols[i]).ToString() + "\n");
			}

		
		} catch (Exception e) {
			Console.WriteLine(e.ToString());
			return;
		}


	}


	private static void on_mtraspose_clicked (object o, EventArgs args)
	{
		double[,] matrix;
		Complex[,] cmatrix;
		bool flag;
		double[,] transpose;
		Complex[,] ctranspose;
		TextBuffer ibuffer = systemsInput.Buffer;
		TextBuffer obuffer = systemsOutput.Buffer;

		try {
			ProcessMatrixInput (out matrix, out cmatrix, out flag, out order);

			if(flag){
				transpose = MatrixAlgebra.Transpose (matrix,  order);
				WriteMatrixToBuffer (ref transpose, order, obuffer);
			}else{
				ctranspose = MatrixAlgebra.Transpose( cmatrix, order );
				WriteMatrixToBuffer (ref ctranspose, order, obuffer);
			}
		
		} catch( Exception e ){
			Console.WriteLine(e.ToString());
		}
	}

	private static void on_madjoint_clicked (object o, EventArgs args)
	{
		double[,] matrix;
		double[,] adjoint;
		Complex[,] cadjoint;
		string[] ipoint = new string[2];
		int[] point = new int[2];
		Complex[,] cmatrix;
		bool flag;
		
		TextBuffer ibuffer = systemsInput.Buffer;
		TextBuffer obuffer = systemsOutput.Buffer;

		try {
			ProcessMatrixInput (out matrix, out cmatrix, out flag, out order);

			if( flag ){
				adjoint = MatrixAlgebra.Adjoint (matrix, order);
				WriteMatrixToBuffer (ref adjoint, order, obuffer);
			}else{
				cadjoint = MatrixAlgebra.Adjoint (cmatrix, order);
				WriteMatrixToBuffer (ref cadjoint, order, obuffer);
			}
		
		} catch( Exception e ){
			Console.WriteLine(e.ToString());
		}
	
	}

	private static void on_minvert_clicked (object o, EventArgs args)
	{
		double[,] matrix;
		double[,] inverse;
		Complex[,] cinverse;
		int order;
		Complex[,] cmatrix;
		bool flag;
		
		TextBuffer ibuffer = systemsInput.Buffer;
		TextBuffer obuffer = systemsOutput.Buffer;

		
		try {
			ProcessMatrixInput (out matrix, out cmatrix, out flag, out order);

		
			if ( flag )
			{
				inverse = LinearAlgebra.Inverse ( matrix );
				WriteMatrixToBuffer (ref inverse, order, obuffer);
			}else{
				cinverse = LinearAlgebra.Inverse( cmatrix );
				WriteMatrixToBuffer ( ref cinverse, order, obuffer);
			}
		
		} catch( Exception e ) {
			Console.WriteLine(e.ToString());
		}

			
	}	
	
	private static void on_mtrace_clicked (object o, EventArgs args)
	{
		TextBuffer ibuffer = systemsInput.Buffer;
		TextBuffer obuffer = systemsOutput.Buffer;

		try {
			ProcessMatrixInput (out matrix, out cmatrix, out flag, out order);
			if( flag ) 
			{
				double tr = MatrixAlgebra.Trace(matrix);
				obuffer.Insert(obuffer.EndIter, "Tr: " + (tr).ToString()+"\n");
			}else{
				Complex tr = MatrixAlgebra.Trace(cmatrix);
				obuffer.Insert(obuffer.EndIter, "Tr: "+ (tr).ToString()+"\n");
			}

		} catch (Exception e) {
			Console.WriteLine (e.ToString());
			return;
		}


	}

	private static void on_mdeterminant_clicked (object o, EventArgs args)
	{
		double[,] matrix;
		int order;
		Complex[,] cmatrix;
		bool flag;

		try {
			ProcessMatrixInput (out matrix, out cmatrix, out flag, out order);
			if( flag )
			{
				systemsOutput.Buffer.Insert (systemsOutput.Buffer.EndIter,
					"Det: " + (LinearAlgebra.Det(matrix)).ToString() + "\n");
			}else{
				Complex Determinant = new Complex();
				Determinant = LinearAlgebra.Det( cmatrix );
				systemsOutput.Buffer.Insert ( systemsOutput.Buffer.EndIter,
						"Det: " + (Determinant.Re).ToString() 
						+ " + " + (Determinant.Im).ToString() + "i\n" );
			}

		
		} catch (Exception e) {
			Console.WriteLine (e.ToString());
			return;
		}

		
	}

	private static void on_mmpv_clicked (object o, EventArgs args)
	{
		double[,] matrix;
		double[] vector;
		double[] prod;
		int order;
		
		TextBuffer ibuffer = systemsInput.Buffer;
		TextBuffer obuffer = systemsOutput.Buffer;

		try {
			ProcessSystemInput (out matrix, out cmatrix, out flag, out vector, out csolVctr);

		
			prod = MatrixAlgebra.Multiply (matrix, vector); 
			WriteVectorToBuffer (ref prod, obuffer);

			}catch( Exception e ){
				Console.WriteLine (e.ToString());
		}
		
	}



	private static void ProcessSystemInput (out double[,] matrix, out Complex[,] cmatrix, 
					out bool flag, out double[] solVctr, out Complex[] csolVctr)
	{
		string[] input;
		string[] numbers;
		TextBuffer ibuffer = systemsInput.Buffer;
		TextBuffer obuffer = systemsOutput.Buffer;

		input = (ibuffer.Text).Split('\n');

		// Validate input
		numbers = input[0].Split(' ');
		if (numbers.Length != input.Length - 1)
			throw new LibregresionException ("Invalid input for a system."
					+"\nPlease consult the documentation.");

		if( ( ibuffer.Text).IndexOf('i') != -1 ) //si hubiera al menos un complejo
		{
			flag = false;
			cmatrix = new Complex[input.Length -1, input.Length -1];
			for( int i = 0; i< input.Length-1; ++i )
			{
				numbers = input[i].Split(' ');
				for ( int j = 0; j< input.Length-1; ++j )
				{
					cmatrix[i,j] = Miscel.ConvertToComplex( numbers[j] );
				}
			}
			
			
			csolVctr = new Complex[numbers.Length];
			numbers = input[input.Length-1].Split(' ');
			for( int k = 0; k< numbers.Length; ++k)
				csolVctr[k] = Miscel.ConvertToComplex( numbers[k] );
			
			// hay que asignar un valor a matrix
			matrix = null;
			solVctr = null;
		}
		else{	
		
			flag = true;
		
			matrix = new double[input.Length - 1, input.Length - 1];
			for (int i=0; i < input.Length - 1; ++i) {
				numbers = input[i].Split(' ');
				for (int j=0; j < numbers.Length; ++j)
					matrix[i,j] = Convert.ToDouble(numbers[j]);
			}
		
			numbers = (input[input.Length - 1]).Split(' ');

			solVctr = new double[numbers.Length];
		
			for (int k=0; k < numbers.Length; ++k)
				solVctr[k] = Convert.ToDouble (numbers[k]);
			

			cmatrix = null;
			csolVctr = null;
		}
	}
	

	private static void ProcessMatrixInput (out double[,] matrix, out Complex[,] cmatrix, 
						out bool flag, out int order)
		//flag indicara si se ha procesado la matriz real o la compleja
		//( true para real, false para compleja )
	{
		string[] input;
		string[] numbers;
		TextBuffer ibuffer = systemsInput.Buffer;
		TextBuffer obuffer = systemsOutput.Buffer;
		
		input = (ibuffer.Text).Split('\n');
		int dim = input.Length;
		
			
		if( (ibuffer.Text).IndexOf('i') != -1)	// si se tratara de numeros conplejos
		{
			numbers = input[0].Split(' ');
			if (numbers.Length != dim)
				throw new LibregresionException ("Invalid input for this matrix operation."
						+"Please consult the documentation.");
			cmatrix = new Complex[dim, dim];
			for( int i = 0; i< dim; i++)
			{
				numbers = input[i].Split(' ');
				for( int j=0; j<dim; j++)
					cmatrix[i,j] = Miscel.ConvertToComplex( numbers[j] );
			}
			
			numbers = (input[dim - 1]).Split(' ');
			order =  dim;
			
			flag = false;
			
			matrix = null; //me obliga a asignarle un valor
			
			return;
			
		}
		else{
			
			// Validate input
			numbers = input[0].Split (' ');
			if (numbers.Length != dim)
				throw new LibregresionException ("Invalid input for this matrix operation. "
						+"Please consult the documentation.");
			
			matrix = new double[dim,dim];
			for (int i=0; i < dim; ++i) {
				numbers = input[i].Split(' ');
				for (int j=0; j < dim; ++j)
					matrix[i,j] = Convert.ToDouble(numbers[j]);
			}
			
			numbers = (input[dim - 1]).Split(' ');
			order =  dim;
			
			flag = true;
			
			cmatrix = null; //me obliga a asignarle un valor
		}

	}

	private static void WriteMatrixToBuffer (ref double[,] matrix, int order, Gtk.TextBuffer buffer)
	{
		for (int i=0; i < order; i++) {
			for (int j=0; j < order; j++)
				buffer.Insert (buffer.EndIter, (matrix[i,j]).ToString()+" ");
			buffer.Insert (buffer.EndIter, Environment.NewLine);
		}
		buffer.Insert ( buffer.EndIter, Environment.NewLine );
	}

	private static void WriteMatrixToBuffer ( ref Complex[,] cmatrix, int order, Gtk.TextBuffer buffer)
	{
		for( int i = 0; i < order; i++) {
			for( int j=0; j < order; j++)
				buffer.Insert ( buffer.EndIter, (cmatrix[i,j]).ToString() + ";  ");
			buffer.Insert ( buffer.EndIter, Environment.NewLine );
		}
		buffer.Insert ( buffer.EndIter, Environment.NewLine );
	}
		

	private static void WriteVectorToBuffer (ref double[] vector, Gtk.TextBuffer buffer)
	{
		for (int i=0; i < order; ++i)
			buffer.Insert (buffer.EndIter, (vector[i]).ToString() + "\n");
		buffer.Insert (buffer.EndIter, Environment.NewLine);
	}



/* 	*****************************************************
 *  		Varios
 *  	*****************************************************/


	private static void on_gcalc_clicked (object o, EventArgs args)
	{
		ProcessStartInfo psi = new ProcessStartInfo ("gnome-calculator");
		psi.UseShellExecute = false;
		gcalc = Process.Start (psi);
	}

	private static void on_save_clicked (object o, EventArgs args)
	{
		save ();
	}

	private static void on_autosave_button_toggled (object o, EventArgs args)
	{
		autosaveButton.Mode = (autosaveButton.Mode) ? false : true;
	}

	private static void on_about_activate (object o, EventArgs args)
	{
		about = new Dialog ();
		about.Title = "About Monumeric";
		about.AddButton (Gtk.Stock.Ok, 1);
		about.Response += new ResponseHandler (on_delete_about);

		VBox vbox = about.VBox;
		HBox hbox = new HBox (false, 4);
		vbox.PackStart (hbox, true, true, 0);

		Gtk.Image icon = icon = new Gtk.Image (Gtk.Stock.DialogInfo, IconSize.Dialog);
		hbox.PackStart (icon, false, false, 0);
		hbox.PackStart (new Label ("Monumeric " + VERSION + "\n"
					+ "A sample GUI for the libregresion LGPL library.\n"
					+ "http://monoevo.sourceforge.net/monumeric/index.html\n"
					+ "(C) 2003-2004, Jaime Anguiano Olarra <jaime@gnome.org>\n"
					+ "  Fabian Seoane <f.seoane@unix.net>" ),
				false, false, 0);
		about.ShowAll ();
	}

	private static void on_delete_about (object o, ResponseArgs args)
	{
		about.Destroy ();
	}
	
	private static void save ()
	{
		uint c = 0;
		string filename = null;
		StreamWriter writer;
		FileStream file;
		filename = ((Gtk.Entry) ui["filename_entry"]).Text;
		if (filename == null)
			return;
		else {
			file = new FileStream (filename, FileMode.Append, FileAccess.Write);
			writer = new StreamWriter (file);
		}
		writer.WriteLine ("Data obtained with Monumeric " + VERSION);
		writer.WriteLine ("Date: " + (DateTime.Now).ToString());
		if (changes[0]) {
			writer.WriteLine ("Iterations used for Gauss-Seidel: " + iters);
			writer.WriteLine ("Order selected: " + (order - 1));
			writer.WriteLine ("X Data: " + xentry.Text);
			writer.WriteLine ("Y Data: " + yentry.Text);
			writer.WriteLine ("\ny(x) = a[0] + a[1]x + a[2]x^2 + ...");
			while (c < regsols.Length)
				writer.WriteLine ("a[" + (c).ToString() + "]" + regsols[c++]);
			changes[0] = false;
		}

		if (changes[1]) {
			writer.WriteLine ("X Data: " + interp_xentry.Text);
			writer.WriteLine ("Y Data: " + interp_yentry.Text);
			writer.WriteLine ("Queried value: " + interp_val_entry.Text);
			writer.WriteLine ("Results: " + interp_results.Text);
		}
		writer.WriteLine ("http://monoevo.sf.net/monumeric/index.html\n");
		writer.Flush ();
		writer.Close ();
		writer = null;
	}


	
}
